package de.fhdw.gaming.ipspiel23.memory;

import java.util.ArrayList;
import java.util.List;

/**
 * implements {@link IGameMemory}.
 * 
 * @param <ROUND_DATA> Round-Data, stored in the memory.
 */
public class GameMemory<ROUND_DATA> implements IGameMemory<ROUND_DATA> {

    /**
     * List where the round-data is stored.
     */
    private final List<ROUND_DATA> data;

    /**
     * capacity of the memory, contains if it's unlimited and if it's the default.
     */
    private final IGameMemoryCapacity memoryCapacity;

    /**
     * Constructor.
     * 
     * @param capacity capacity.
     */
    public GameMemory(final IGameMemoryCapacity capacity) {
        this.memoryCapacity = capacity;
        this.data = new ArrayList<>();
    }

    @Override
    public int size() {
        return this.data.size();
    }

    @Override
    public int capacity() {
        return this.memoryCapacity.getCapacity();
    }

    @Override
    public void add(final ROUND_DATA item) {
        if (!this.memoryCapacity.isUnlimited() && this.data.size() >= capacity()) {
            this.data.remove(0);
        }
        this.data.add(item);
    }

    @Override
    public void clear() {
        this.data.clear();
    }

    @Override
    /**
     * Returns the round data at the specified index.
     * 
     * @param index the index of the round data to retrieve.
     * @param fromEnd if true, retrieves the round data from the end of the buffer.
     * @return the round data at the specified index, or null if the buffer is empty.
     */
    public ROUND_DATA getRound(final int index, final boolean fromEnd) {
        if (data.isEmpty()) {
            return null;
        }
        if (fromEnd) {
            return data.get(data.size() - index - 1);
        } else {
            return data.get(index);
        }
    }

    /**
     * Returns the round data at the specified index.
     * Counted from the end of the buffer.
     * 
     * @param index the index of the round data to retrieve.
     * @return the round data at the specified index, or null if the buffer is empty.
     */
    @Override
    public ROUND_DATA getRound(final int index) {
        return getRound(index, true);
    }

    /**
     * Returns the latest round data in the buffer.
     * 
     * @return the latest round data in the buffer, or null if the buffer is empty.
     */
    @Override
    public ROUND_DATA getLatestRound() {
        return getRound(0, true);
    }
}
